import React, {
  useCallback,
  useContext,
  useEffect,
  useMemo,
  useState,
} from "react";
import axios, { type AxiosInstance } from "axios";
import type { IBusiness } from "../typings/business";
import analytics from "../utils/segment";
import { PaperProvider } from "react-native-paper";
import { darkTheme, lightTheme } from "../theme";
import type { ThemeProp } from "react-native-paper/src/types";
import FlashMessage from "react-native-flash-message";
import { SafeAreaProvider, SafeAreaView } from "react-native-safe-area-context";

type LMSContextProps = {
  TOKEN: string;
  BASE_URL: string;
  axiosInstance: AxiosInstance;
  EMAIL?: string;
  setEmail?(email: string): void;
  userDetail?: IBusiness;
};

type Props = {
  children: React.ReactNode;
  token: string;
  base_url: string;
  theme?: "light" | "dark";
  palette?: ThemeProp;
  email?: string;
  //fonts?: string[];
};

const LMSContext = React.createContext<LMSContextProps>({} as LMSContextProps);

export const LMSProvider: React.FC<Props> = ({
  children,
  token,
  base_url,
  theme = "light",
  palette,
  email,
  //fonts,
}) => {
  const [userEmail, setUserEmail] = useState(email);
  const [userDetail, setUserDetail] = useState<IBusiness>();

  const axiosInstance = axios.create({
    baseURL: base_url,
    headers: {
      Authorization: `Bearer ${token}`,
    },
  });

  const fetchData = useCallback(async () => {
    try {
      const { data }: { data: IBusiness } = await axios.get(
        `${base_url}v1/external-lending/sme/users?email=${userEmail}`,
        {
          headers: {
            Authorization: `Bearer ${token}`,
          },
        },
      );

      setUserDetail(data);
      analytics.identify(data.kyb.id);
    } catch (e: any) {
      //
    }
  }, [base_url, token, userEmail]);

  const value: LMSContextProps = useMemo(() => {
    return {
      TOKEN: token,
      BASE_URL: base_url,
      axiosInstance: axiosInstance,
      EMAIL: userEmail,
      setEmail: (email: string) => setUserEmail(email),
      userDetail: userDetail,
    };
  }, [token, base_url, axiosInstance, userEmail, userDetail]);

  useEffect(() => {
    if (userEmail) {
      fetchData();
    } else if (userEmail === "") {
      setUserDetail(undefined);
    }
  }, [userEmail, fetchData]);

  return (
    <SafeAreaProvider>
      <SafeAreaView>
        <LMSContext.Provider value={value}>
          <PaperProvider
            theme={
              palette ? palette : theme === "light" ? lightTheme : darkTheme
            }
          >
            {children}
            <FlashMessage position="top" />
          </PaperProvider>
        </LMSContext.Provider>
      </SafeAreaView>
    </SafeAreaProvider>
  );
};

export const useLMS = () => useContext(LMSContext);
